==> pydantic_llm_tester-0.1.12.tar.gz: pydantic_llm_tester-0.1.12/tests/test_refactored_accuracy.py <==
"""
Tests for the *refactored* LLMTester accuracy calculation logic.

These tests define the expected behavior of the improved accuracy metrics,
including granular string matching, list comparison options, numerical tolerance,
and field weighting. They will likely fail until the _calculate_accuracy
method (or its replacement) is refactored.
"""

import pytest
from datetime import date

# Placeholder for the refactored accuracy calculation function/method
# We might need to import it differently after refactoring.
# For now, assume it's accessible via a tester instance or as a standalone function.
from src import LLMTester
tester_instance = LLMTester(providers=[]) # Minimal instance

# --- Test Cases for Refactored Accuracy Logic ---

# 1. Granular String Similarity (e.g., using Levenshtein distance)
# Assuming a function like calculate_accuracy(actual, expected, string_similarity_threshold=75)
# Or the function returns a similarity score directly. Let's assume it returns % accuracy.

def test_refactored_string_high_similarity():
    """Test high string similarity results in high score (e.g., > 80%)."""
    actual = {"title": "Software Enginer"} # Typo
    expected = {"title": "Software Engineer"}
    # Expected: High score, e.g., ~90-95% based on Levenshtein distance
    # This assertion will need adjustment based on the chosen algorithm/scaling
    accuracy = tester_instance._calculate_accuracy(actual, expected) # Or refactored_calculate_accuracy(...)
    assert accuracy > 80.0
    assert accuracy < 100.0

def test_refactored_string_low_similarity():
    """Test low string similarity results in low score."""
    actual = {"description": "A red car"}
    expected = {"description": "Some blue bicycle"}
    # Expected: Low score, e.g., < 30%
    accuracy = tester_instance._calculate_accuracy(actual, expected)
    assert accuracy < 30.0

# 2. List Comparison Options

def test_refactored_list_order_insensitive():
    """Test list comparison ignoring order."""
    actual = {"skills": ["python", "sql", "react"]}
    expected = {"skills": ["react", "python", "sql"]}
    # Assuming an option like calculate_accuracy(..., list_comparison='set')
    # Or the default behavior changes. Expected: 100%
    # Need to pass the list_comparison_mode option
    accuracy = tester_instance._calculate_accuracy(actual, expected, list_comparison_mode='set_similarity')
    assert accuracy == 100.0

def test_refactored_list_item_similarity():
    """Test list comparison using granular similarity for items."""
    actual = {"features": ["Fast procesor", "Large screen"]} # Typo in "processor"
    expected = {"features": ["Fast processor", "Large screen"]}
    # Expected: Item 0 has high similarity (>80%), Item 1 is 100%. Overall > 90%.
    # This requires the list comparison to recursively use the string similarity logic.
    # Need to pass the list_comparison_mode option
    accuracy = tester_instance._calculate_accuracy(actual, expected, list_comparison_mode='ordered_similarity')
    # Recalculate expected score based on implemented scaling:
    # Item 0 fuzz.ratio ~96.3%. Scaled score = (96.3-80)/(100-80) = 0.815
    # Item 1 score = 1.0
    # Average = (0.815 + 1.0) / 2 = 0.9075 = 90.75%
    assert accuracy == pytest.approx(90.75, abs=0.1) # Corrected assertion

# 3. Numerical Tolerance

def test_refactored_numerical_within_tolerance():
    """Test numerical match within a specified tolerance."""
    actual = {"price": 102.50}
    expected = {"price": 100.00}
    # Assuming an option like calculate_accuracy(..., numerical_tolerance=0.05) for 5%
    # 102.50 is within 5% of 100.00. Expected: 100%
    accuracy = tester_instance._calculate_accuracy(actual, expected, numerical_tolerance=0.05)
    assert accuracy == 100.0

def test_refactored_numerical_outside_tolerance():
    """Test numerical mismatch outside a specified tolerance."""
    actual = {"quantity": 110}
    expected = {"quantity": 100}
    # Assuming 5% tolerance. 110 is outside 5% of 100. Expected: 0% (or reduced score)
    accuracy = tester_instance._calculate_accuracy(actual, expected, numerical_tolerance=0.05)
    assert accuracy == 0.0 # Or assert accuracy < 100.0 depending on desired behavior

# 4. Field Weighting

def test_refactored_field_weighting():
    """Test accuracy calculation with weighted fields."""
    actual = {"id": "abc", "critical_field": "wrong", "optional_field": "match"}
    expected = {"id": "abc", "critical_field": "correct", "optional_field": "match"}
    weights = {"critical_field": 3.0, "optional_field": 1.0} # 'id' defaults to 1.0
    # Points:
    # id: 1.0 * 1.0 = 1.0 point earned / 1.0 possible
    # critical_field: 0.0 * 3.0 = 0.0 points earned / 3.0 possible (assuming low similarity score < threshold)
    # optional_field: 1.0 * 1.0 = 1.0 point earned / 1.0 possible
    # Total earned = 1.0 + 0.0 + 1.0 = 2.0
    # Total possible = 1.0 + 3.0 + 1.0 = 5.0
    # Expected accuracy = 2.0 / 5.0 = 40%
    accuracy = tester_instance._calculate_accuracy(actual, expected, field_weights=weights)
    assert accuracy == 40.0

# 5. Detailed Mismatch Information (Testing the structure of the return value)
# Note: The current refactored function only returns the float accuracy.
# This test would require further modification of the return value.
# Skipping this test for now as it tests a feature not implemented in the current refactor.
@pytest.mark.skip(reason="Refactored function currently only returns float accuracy")
def test_refactored_return_detailed_mismatches():
    """Test that the refactored function returns detailed mismatch info."""
    actual = {"name": "Test", "value": 95}
    expected = {"name": "Testt", "value": 100}
    # Expected return structure might be:
    # {
    #   'accuracy': 75.0, # Example score
    #   'field_details': {
    #     'name': {'match': False, 'score': 0.8, 'reason': 'String similarity 80%'},
    #     'value': {'match': False, 'score': 0.0, 'reason': 'Value mismatch outside tolerance'}
    #   }
    # }
    result = tester_instance._calculate_accuracy(actual, expected) # Or refactored_calculate_accuracy(...)
    assert isinstance(result, dict)
    assert 'accuracy' in result
    assert 'field_details' in result
    assert 'name' in result['field_details']
    assert 'value' in result['field_details']
    assert 'match' in result['field_details']['name']
    assert 'score' in result['field_details']['name']
    assert 'reason' in result['field_details']['name']

==> pydantic_llm_tester-0.1.16.tar.gz: pydantic_llm_tester-0.1.16/tests/test_refactored_accuracy.py <==
"""
Tests for the *refactored* LLMTester accuracy calculation logic.

These tests define the expected behavior of the improved accuracy metrics,
including granular string matching, list comparison options, numerical tolerance,
and field weighting. They will likely fail until the _calculate_accuracy
method (or its replacement) is refactored.
"""

import pytest
from datetime import date

# Placeholder for the refactored accuracy calculation function/method
# We might need to import it differently after refactoring.
# For now, assume it's accessible via a tester instance or as a standalone function.
from pydantic_llm_tester import LLMTester
tester_instance = LLMTester(providers=[]) # Minimal instance

# --- Test Cases for Refactored Accuracy Logic ---

# 1. Granular String Similarity (e.g., using Levenshtein distance)
# Assuming a function like calculate_accuracy(actual, expected, string_similarity_threshold=75)
# Or the function returns a similarity score directly. Let's assume it returns % accuracy.

def test_refactored_string_high_similarity():
    """Test high string similarity results in high score (e.g., > 80%)."""
    actual = {"title": "Software Enginer"} # Typo
    expected = {"title": "Software Engineer"}
    # Expected: High score, e.g., ~90-95% based on Levenshtein distance
    # This assertion will need adjustment based on the chosen algorithm/scaling
    accuracy = tester_instance._calculate_accuracy(actual, expected) # Or refactored_calculate_accuracy(...)
    assert accuracy > 80.0
    assert accuracy < 100.0

def test_refactored_string_low_similarity():
    """Test low string similarity results in low score."""
    actual = {"description": "A red car"}
    expected = {"description": "Some blue bicycle"}
    # Expected: Low score, e.g., < 30%
    accuracy = tester_instance._calculate_accuracy(actual, expected)
    assert accuracy < 30.0

# 2. List Comparison Options

def test_refactored_list_order_insensitive():
    """Test list comparison ignoring order."""
    actual = {"skills": ["python", "sql", "react"]}
    expected = {"skills": ["react", "python", "sql"]}
    # Assuming an option like calculate_accuracy(..., list_comparison='set')
    # Or the default behavior changes. Expected: 100%
    # Need to pass the list_comparison_mode option
    accuracy = tester_instance._calculate_accuracy(actual, expected, list_comparison_mode='set_similarity')
    assert accuracy == 100.0

def test_refactored_list_item_similarity():
    """Test list comparison using granular similarity for items."""
    actual = {"features": ["Fast procesor", "Large screen"]} # Typo in "processor"
    expected = {"features": ["Fast processor", "Large screen"]}
    # Expected: Item 0 has high similarity (>80%), Item 1 is 100%. Overall > 90%.
    # This requires the list comparison to recursively use the string similarity logic.
    # Need to pass the list_comparison_mode option
    accuracy = tester_instance._calculate_accuracy(actual, expected, list_comparison_mode='ordered_similarity')
    # Recalculate expected score based on implemented scaling:
    # Item 0 fuzz.ratio ~96.3%. Scaled score = (96.3-80)/(100-80) = 0.815
    # Item 1 score = 1.0
    # Average = (0.815 + 1.0) / 2 = 0.9075 = 90.75%
    assert accuracy == pytest.approx(90.75, abs=0.1) # Corrected assertion

# 3. Numerical Tolerance

def test_refactored_numerical_within_tolerance():
    """Test numerical match within a specified tolerance."""
    actual = {"price": 102.50}
    expected = {"price": 100.00}
    # Assuming an option like calculate_accuracy(..., numerical_tolerance=0.05) for 5%
    # 102.50 is within 5% of 100.00. Expected: 100%
    accuracy = tester_instance._calculate_accuracy(actual, expected, numerical_tolerance=0.05)
    assert accuracy == 100.0

def test_refactored_numerical_outside_tolerance():
    """Test numerical mismatch outside a specified tolerance."""
    actual = {"quantity": 110}
    expected = {"quantity": 100}
    # Assuming 5% tolerance. 110 is outside 5% of 100. Expected: 0% (or reduced score)
    accuracy = tester_instance._calculate_accuracy(actual, expected, numerical_tolerance=0.05)
    assert accuracy == 0.0 # Or assert accuracy < 100.0 depending on desired behavior

# 4. Field Weighting

def test_refactored_field_weighting():
    """Test accuracy calculation with weighted fields."""
    actual = {"id": "abc", "critical_field": "wrong", "optional_field": "match"}
    expected = {"id": "abc", "critical_field": "correct", "optional_field": "match"}
    weights = {"critical_field": 3.0, "optional_field": 1.0} # 'id' defaults to 1.0
    # Points:
    # id: 1.0 * 1.0 = 1.0 point earned / 1.0 possible
    # critical_field: 0.0 * 3.0 = 0.0 points earned / 3.0 possible (assuming low similarity score < threshold)
    # optional_field: 1.0 * 1.0 = 1.0 point earned / 1.0 possible
    # Total earned = 1.0 + 0.0 + 1.0 = 2.0
    # Total possible = 1.0 + 3.0 + 1.0 = 5.0
    # Expected accuracy = 2.0 / 5.0 = 40%
    accuracy = tester_instance._calculate_accuracy(actual, expected, field_weights=weights)
    assert accuracy == 40.0

# 5. Detailed Mismatch Information (Testing the structure of the return value)
# Note: The current refactored function only returns the float accuracy.
# This test would require further modification of the return value.
# Skipping this test for now as it tests a feature not implemented in the current refactor.
@pytest.mark.skip(reason="Refactored function currently only returns float accuracy")
def test_refactored_return_detailed_mismatches():
    """Test that the refactored function returns detailed mismatch info."""
    actual = {"name": "Test", "value": 95}
    expected = {"name": "Testt", "value": 100}
    # Expected return structure might be:
    # {
    #   'accuracy': 75.0, # Example score
    #   'field_details': {
    #     'name': {'match': False, 'score': 0.8, 'reason': 'String similarity 80%'},
    #     'value': {'match': False, 'score': 0.0, 'reason': 'Value mismatch outside tolerance'}
    #   }
    # }
    result = tester_instance._calculate_accuracy(actual, expected) # Or refactored_calculate_accuracy(...)
    assert isinstance(result, dict)
    assert 'accuracy' in result
    assert 'field_details' in result
    assert 'name' in result['field_details']
    assert 'value' in result['field_details']
    assert 'match' in result['field_details']['name']
    assert 'score' in result['field_details']['name']
    assert 'reason' in result['field_details']['name']
